/***************************************************************************
                          stuff.cpp  -  description
                             -------------------
    begin                : Sun May 6 2001
    copyright            : (C) 2001 by Stefan Winter
    email                : mail@stefan-winter.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stuff.h"
#include <qdir.h>
#include <qfile.h>
#include <qstringlist.h>
#include <arts/artsflow.h>
#include <arts/connect.h>
#include <arts/iomanager.h>
#include <arts/referenceclean.h>
#include <iostream>
#include <string>
#include <klocale.h>
#include <kprocio.h>
#include <kdebug.h>
#include <qstring.h>
#include <arpa/inet.h>

extern "C" {
#include <iwlib.h>
}

/* ====== Taken from IWCONFIG.C ====== */

static int
get_info(int			skfd,
	 char *			ifname,
	 struct wireless_info *	info)
{
  struct iwreq		wrq;

  memset((char *) info, 0, sizeof(struct wireless_info));

  /* Get wireless name */
  if(iw_get_ext(skfd, ifname, SIOCGIWNAME, &wrq) < 0)
    {
      /* If no wireless name : no wireless extensions */
      /* But let's check if the interface exists at all */
      struct ifreq ifr;

      strcpy(ifr.ifr_name, ifname);
      if(ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0)
	return(-ENODEV);
      else
	return(-ENOTSUP);
    }
  else
    {
      strncpy(info->name, wrq.u.name, IFNAMSIZ);
      info->name[IFNAMSIZ] = '\0';
    }

  /* Get ranges */
  if(iw_get_range_info(skfd, ifname, &(info->range)) >= 0)
    info->has_range = 1;

  /* Get network ID */
  if(iw_get_ext(skfd, ifname, SIOCGIWNWID, &wrq) >= 0)
    {
      info->has_nwid = 1;
      memcpy(&(info->nwid), &(wrq.u.nwid), sizeof(iwparam));
    }

  /* Get frequency / channel */
  if(iw_get_ext(skfd, ifname, SIOCGIWFREQ, &wrq) >= 0)
    {
      info->has_freq = 1;
      info->freq = iw_freq2float(&(wrq.u.freq));
    }

  /* Get sensitivity */
  if(iw_get_ext(skfd, ifname, SIOCGIWSENS, &wrq) >= 0)
    {
      info->has_sens = 1;
      memcpy(&(info->sens), &(wrq.u.sens), sizeof(iwparam));
    }

  /* Get encryption information */
  wrq.u.data.pointer = (caddr_t) info->key;
  wrq.u.data.length = IW_ENCODING_TOKEN_MAX;
  wrq.u.data.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWENCODE, &wrq) >= 0)
    {
      info->has_key = 1;
      info->key_size = wrq.u.data.length;
      info->key_flags = wrq.u.data.flags;
    }

  /* Get ESSID */
  wrq.u.essid.pointer = (caddr_t) info->essid;
  wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
  wrq.u.essid.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWESSID, &wrq) >= 0)
    {
      info->has_essid = 1;
      info->essid_on = wrq.u.data.flags;
    }

  /* Get AP address */
  if(iw_get_ext(skfd, ifname, SIOCGIWAP, &wrq) >= 0)
    {
      info->has_ap_addr = 1;
      memcpy(&(info->ap_addr), &(wrq.u.ap_addr), sizeof (sockaddr));
    }

  /* Get NickName */
  wrq.u.essid.pointer = (caddr_t) info->nickname;
  wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
  wrq.u.essid.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWNICKN, &wrq) >= 0)
    if(wrq.u.data.length > 1)
      info->has_nickname = 1;

  /* Get bit rate */
  if(iw_get_ext(skfd, ifname, SIOCGIWRATE, &wrq) >= 0)
    {
      info->has_bitrate = 1;
      memcpy(&(info->bitrate), &(wrq.u.bitrate), sizeof(iwparam));
    }

  /* Get RTS threshold */
  if(iw_get_ext(skfd, ifname, SIOCGIWRTS, &wrq) >= 0)
    {
      info->has_rts = 1;
      memcpy(&(info->rts), &(wrq.u.rts), sizeof(iwparam));
    }

  /* Get fragmentation threshold */
  if(iw_get_ext(skfd, ifname, SIOCGIWFRAG, &wrq) >= 0)
    {
      info->has_frag = 1;
      memcpy(&(info->frag), &(wrq.u.frag), sizeof(iwparam));
    }

  /* Get operation mode */
  if(iw_get_ext(skfd, ifname, SIOCGIWMODE, &wrq) >= 0)
    {
      info->mode = wrq.u.mode;
      if((info->mode < IW_NUM_OPER_MODE) && (info->mode >= 0))
	info->has_mode = 1;
    }

  /* Get Power Management settings */
  wrq.u.power.flags = 0;
  if(iw_get_ext(skfd, ifname, SIOCGIWPOWER, &wrq) >= 0)
    {
      info->has_power = 1;
      memcpy(&(info->power), &(wrq.u.power), sizeof(iwparam));
    }

#if WIRELESS_EXT > 9
  /* Get Transmit Power */
  if(iw_get_ext(skfd, ifname, SIOCGIWTXPOW, &wrq) >= 0)
    {
      info->has_txpower = 1;
      memcpy(&(info->txpower), &(wrq.u.txpower), sizeof(iwparam));
    }
#endif

#if WIRELESS_EXT > 10
  /* Get retry limit/lifetime */
  if(iw_get_ext(skfd, ifname, SIOCGIWRETRY, &wrq) >= 0)
    {
      info->has_retry = 1;
      memcpy(&(info->retry), &(wrq.u.retry), sizeof(iwparam));
    }
#endif	/* WIRELESS_EXT > 10 */

  /* Get stats */
  if(iw_get_stats(skfd, ifname, &(info->stats)) >= 0)
    {
      info->has_stats = 1;
    }

  return(0);
};

/* ====== END ====== */


int operator == (const deviceinfo & eins, const deviceinfo & zwei) {

/*	There are several levels of difference, enumerated with integers:
	0		no difference
	1		strength different
    2		status different
	4		operation mode different
	8		connection speed different
*/
  int result = 0;
  if (eins.bitrate.value != zwei.bitrate.value)
    result += 8;
  if ((eins.has_mode != zwei.has_mode) || (eins.mode != zwei.mode))
    result += 4;
  char stupid[255];
  if ((eins.essid != zwei.essid) || (iw_pr_ether (stupid,(unsigned char *) eins.ap_addr.sa_data) != iw_pr_ether (stupid,(unsigned char *) zwei.ap_addr.sa_data)) || (eins.freq != zwei.freq) || (eins.ip_addr != zwei.ip_addr) || (eins.has_key !=zwei.has_key) || (eins.key_size!=zwei.key_size)||(eins.key_flags!=zwei.key_flags))
    result += 2;
  if ((eins.qual.qual != zwei.qual.qual) || (eins.has_freq != zwei.has_freq))
    result += 1;
  return result;
};

Interface_Talker::Interface_Talker(QString iface){
    device_information.has_frequency=false;
    device_information.frequency=0.;
    device_information.has_mode=false;
    device_information.mode=0;
    device_information.has_key=0;
    device_information.key="";
    device_information.key_size=0;
    device_information.key_flags=0;
    device_information.essid="";
    device_information.access_point_address="";
    device_information.ip_address="";
    device_information.bitrate=0.;
    device_information.interface_name=iface;
    device_information.socket=-1;
    device_information.has_range=false;
    for (int i=0;i<MAX_HISTORY;i++) {
      statistical_data.sigLevel[i]=0;
      statistical_data.noiseLevel[i]=0;
      statistical_data.qual[i]=0;
      statistical_data.valid[i]=false;
      statistical_data.current=0;
    };
};

int Interface_Talker_wirelessextensions::open_device_socket() {
  device_information.socket = iw_sockets_open();
  return device_information.socket;
};

void Interface_Talker_wirelessextensions::poll_device_info(){
  kdDebug() << "unimplemented\n";
};

bool Interface_Talker_wirelessextensions::give_device_freq(float &freq){
  kdDebug() << "unimplemented\n";
  return false;
};

bool Interface_Talker_wirelessextensions::give_mode(int &mode){
  kdDebug() << "unimplemented\n";
  return false;
};

bool Interface_Talker_wirelessextensions::give_key(QString &key,int size,int flags){
  kdDebug() << "unimplemented\n";
  return false;
};

QString Interface_Talker_wirelessextensions::give_essid(){
  kdDebug() << "unimplemented\n";
  return "";
};

bool Interface_Talker_wirelessextensions::give_AP_info(QString &mac, QString &ip){
  kdDebug() << "unimplemented\n";
  return false;
};

double Interface_Talker_wirelessextensions::give_bitrate(){
  kdDebug() << "unimplemented\n";
  return 0.;
};

QString Interface_Talker_wirelessextensions::give_interface_name(){
  kdDebug() << "unimplemented\n";
  return "";
};

bool Interface_Talker_wirelessextensions::give_current_quality(int &sig, int &noi, int &qua){
  kdDebug() << "unimplemented\n";
  return false;
};

void
sinus_wave (double frequency)
{
  using namespace Arts;
  StdIOManager *limiter = new StdIOManager;
  Dispatcher dispatcher (limiter);
  MyTimeNotify *zeit = new MyTimeNotify (&dispatcher);
  Synth_FREQUENCY freq;
  Synth_WAVE_SIN sin;
  Synth_PLAY play;
  setValue (freq, frequency);
  connect (freq, sin);
  connect (sin, play, "invalue_left");
  freq.start ();
  sin.start ();
  play.start ();
  limiter->addTimer (250, zeit);
  dispatcher.run ();
  play.stop ();
  sin.stop ();
  freq.stop ();
};

void
MyTimeNotify::notifyTime ()
{
  test->terminate ();
};

QString
whois (char *MAC_ADR, QStringList APList)
{
  for (QStringList::Iterator it = APList.begin (); it != APList.end (); (it++)++)
    {
      if ((*it) == (QString) MAC_ADR)
	return *(++it);
    };
  return i18n ("UNKNOWN");
};

// bool already_warned = false;

QString
return_info (deviceinfo * transfer, bool fetch_ip)
{
  wireless_info *tempic1 = new (wireless_info);
  iwstats *tempic2 = new (iw_statistics);
  if (transfer->used_interface == "")
    {
      if (!already_warned) kdDebug() << "Autodetecting...\n";
      QFile procnetdev ("/proc/net/dev");
      procnetdev.open (IO_ReadOnly);
      QStringList liste;
      QString temp = "";
      QString temptrash = "X";
      int end = 1;
      while ((end != 0) && (temp != temptrash))
	{
	  end = procnetdev.readLine (temp, 9999);
	  temptrash = temp;
	  temp.truncate (temp.find (":") > 0 ? temp.find (":") : 0);
	  temp = temp.stripWhiteSpace ();
	  if ((end != 0) && (temp != ""))
	    liste += temp;
	};
      for (QStringList::Iterator it = liste.begin (); it != liste.end (); ++it)
	{
	  if (!already_warned) kdDebug() << "[ " << *it << " ] \n";
    int result = get_info (transfer->skfd, (char*)(*it).latin1(), tempic1);
	  if ((result != -ENODEV) && (result != -ENOTSUP))
	    {
	      iw_get_stats (transfer->skfd, (char*)(*it).latin1(), tempic2);
	      kdDebug() << "Found!\n";
        already_warned=false;
	      transfer->has_freq = tempic1->has_freq;
	      transfer->freq = tempic1->freq;
	      transfer->mode = tempic1->mode;
	      transfer->has_mode = tempic1->has_mode;
	      transfer->has_key = tempic1->has_key;
	      char *test = (char *) tempic1->key;
	      transfer->key = test;
	      transfer->key_size = tempic1->key_size;
	      transfer->key_flags = tempic1->key_flags;
	      transfer->essid = tempic1->essid;
	      transfer->ap_addr = tempic1->ap_addr;
	      transfer->bitrate = tempic1->bitrate;
	      transfer->used_interface = *it;
	      transfer->qual = tempic2->qual;
	      transfer->has_range = tempic1->has_range;
	      transfer->range = tempic1->range;
	      if (fetch_ip) local_ip (transfer);
	      delete tempic1;
	      delete tempic2;
	      return *it;
	    };
	};
      if (!already_warned) kdDebug() << "\nFound nothing. Scanning will continue every second without further notice.\n";
      already_warned=true;
      transfer->qual.qual = 0;
      transfer->qual.noise = 154;
      transfer->qual.level = 154;
      transfer->has_freq = tempic1->has_freq;
      transfer->freq = tempic1->freq;
      transfer->mode = tempic1->mode;
      transfer->has_mode = tempic1->has_mode;
      transfer->has_key = tempic1->has_key;
      char *test = (char *) tempic1->key;
      transfer->key = test;
      transfer->key_size = tempic1->key_flags;
      transfer->key_flags = tempic1->key_flags;
      transfer->essid = tempic1->essid;
      transfer->ap_addr = tempic1->ap_addr;
      transfer->bitrate = tempic1->bitrate;
      transfer->used_interface = "";
      transfer->qual = tempic2->qual;
      transfer->has_range = tempic1->has_range;
      transfer->range = tempic1->range;
      if (fetch_ip) local_ip (transfer);
      delete tempic1;
      delete tempic2;
      return transfer->used_interface;
    }
  else
    {
    int result = get_info (transfer->skfd, (char*)transfer->used_interface.latin1 (), tempic1);
    if ((result != -ENODEV) && (result != -ENOTSUP))
	{
	  iw_get_stats (transfer->skfd, (char*)transfer->used_interface.latin1 (), tempic2);
	  transfer->has_freq = tempic1->has_freq;
	  transfer->freq = tempic1->freq;
	  transfer->mode = tempic1->mode;
	  transfer->has_mode = tempic1->has_mode;
	  transfer->has_key = tempic1->has_key;
	  char *test = (char *) tempic1->key;
	  transfer->key = test;
	  transfer->key_size = tempic1->key_flags;
	  transfer->key_flags = tempic1->key_flags;
	  transfer->essid = tempic1->essid;
	  transfer->ap_addr = tempic1->ap_addr;
	  transfer->bitrate = tempic1->bitrate;
	  transfer->qual = tempic2->qual;
	  transfer->has_range = tempic1->has_range;
	  transfer->range = tempic1->range;
	  if (fetch_ip) local_ip (transfer);
	  delete tempic1;
	  delete tempic2;
	  return transfer->used_interface;
	}
      else
	{
	  kdDebug() << "Looks like the card has been detached. Switching back to autodetection.\n";
	  transfer->qual.qual = 0;
	  transfer->qual.level = 154;
	  transfer->qual.noise = 154;
	  transfer->has_freq = tempic1->has_freq;
	  transfer->freq = tempic1->freq;
	  transfer->mode = tempic1->mode;
	  transfer->has_mode = tempic1->has_mode;
	  transfer->has_key = tempic1->has_key;
	  char *test = (char *) tempic1->key;
	  transfer->key = test;
	  transfer->key_size = tempic1->key_flags;
	  transfer->key_flags = tempic1->key_flags;
	  transfer->essid = tempic1->essid;
	  transfer->ap_addr = tempic1->ap_addr;
	  transfer->bitrate = tempic1->bitrate;
	  transfer->used_interface = "";
	  transfer->qual = tempic2->qual;
	  transfer->has_range = tempic1->has_range;
	  transfer->range = tempic1->range;
	  if (fetch_ip) local_ip (transfer);
    already_warned=false;
	  delete tempic1;
	  delete tempic2;
	  return "";
	};
    }
};

void local_ip (deviceinfo * device)
{
  struct sockaddr *sa;
  struct sockaddr_in *sin;
  struct ifreq ifr;
  /* Copy the interface name into the buffer */
  strncpy (ifr.ifr_name, device->used_interface.latin1 (), IFNAMSIZ);

  if (ioctl (device->skfd, SIOCGIFADDR, &ifr) == -1)
    {
      device->ip_addr = "unavailable";
      return;
    }
  /* Now the buffer will contain the information we requested */
  sa = (struct sockaddr *) &(ifr.ifr_addr);
  if (sa->sa_family == AF_INET)
    {
      sin = (struct sockaddr_in *) sa;
      device->ip_addr = (QString) inet_ntoa (sin->sin_addr);
      return;
    };
  device->ip_addr = "unavailable";
}



